/// <reference types="node" />
import { GameSettings, BaseGameDataMessage, BaseRootMessage, ComponentSpawnData, PacketDecoder, AllGameSettings } from "@skeldjs/protocol";
import { GameOverReason, SpawnType } from "@skeldjs/constant";
import { ExtractEventTypes } from "@skeldjs/events";
import { AirshipStatus, AprilShipStatus, CustomNetworkTransform, GameData, GameDataEvents, MiraShipStatus, LobbyBehaviour, LobbyBehaviourEvents, MeetingHud, MeetingHudEvents, PolusShipStatus, PlayerControl, PlayerPhysics, SkeldShipStatus, ShipStatusEvents, VoteBanSystem, VoteBanSystemEvents, PlayerIDResolvable, InnerShipStatus } from "./objects";
import { Heritable, HeritableEvents, NetworkableConstructor } from "./Heritable";
import { Networkable, NetworkableEvents } from "./Networkable";
import { PlayerData, PlayerDataEvents } from "./PlayerData";
import { HostableOptions } from "./misc/HostableOptions";
import { RoomFixedUpdateEvent, RoomGameEndEvent, RoomGameStartEvent, RoomSetPrivacyEvent } from "./events";
export declare type RoomID = string | number;
export declare type PlayerDataResolvable = number | PlayerData | PlayerControl | PlayerPhysics | CustomNetworkTransform;
export declare type PrivacyType = "public" | "private";
export interface SpawnObject {
    type: number;
    ownerid: number;
    flags: number;
    components: Networkable<any, any>[];
}
export declare type AnyNetworkable<RoomType extends Hostable = Hostable> = AirshipStatus<RoomType> | AprilShipStatus<RoomType> | CustomNetworkTransform<RoomType> | GameData<RoomType> | MiraShipStatus<RoomType> | LobbyBehaviour<RoomType> | MeetingHud<RoomType> | PolusShipStatus<RoomType> | PlayerControl<RoomType> | PlayerPhysics<RoomType> | SkeldShipStatus<RoomType> | VoteBanSystem<RoomType>;
export declare type HostableEvents<RoomType extends Hostable = Hostable> = HeritableEvents<RoomType> & PlayerDataEvents<RoomType> & GameDataEvents<RoomType> & LobbyBehaviourEvents<RoomType> & MeetingHudEvents<RoomType> & ShipStatusEvents<RoomType> & VoteBanSystemEvents<RoomType> & ExtractEventTypes<[
    RoomGameStartEvent<RoomType>,
    RoomGameEndEvent<RoomType>,
    RoomFixedUpdateEvent<RoomType>,
    RoomSetPrivacyEvent<RoomType>
]>;
export declare type GetHostableEvents<T extends Hostable<HostableEvents>> = T extends Hostable<infer X> ? X : never;
/**
 * Represents an object capable of hosting games.
 *
 * See {@link HostableEvents} for events to listen to.
 */
export declare class Hostable<T extends HostableEvents = any> extends Heritable<T> {
    options: HostableOptions;
    room: this;
    /**
     * A list of all objects in the room.
     */
    objectList: Networkable[];
    /**
     * The players in the room.
     */
    players: Map<number, PlayerData<this>>;
    /**
     * The networkable components in the room.
     */
    netobjects: Map<number, Networkable<any, NetworkableEvents, this>>;
    registeredPrefabs: Map<SpawnType, typeof Networkable[]>;
    /**
     * The current message stream to be sent to the server on fixed update.
     */
    stream: BaseGameDataMessage[];
    /**
     * The code of the room.
     */
    code: number;
    /**
     * The ID of the host of the room.
     */
    hostid: number;
    protected _incr_netid: number;
    /**
     * The settings of the room.
     */
    settings: GameSettings;
    /**
     * The current start counter for the room.
     */
    counter: number;
    /**
     * The privacy state of the room.
     */
    privacy: PrivacyType;
    /**
     * The packet decoder.
     */
    decoder: PacketDecoder;
    shipstatus: InnerShipStatus<this> | undefined;
    meetinghud: MeetingHud<this> | undefined;
    lobbybehaviour: LobbyBehaviour<this> | undefined;
    gamedata: GameData<this> | undefined;
    votebansystem: VoteBanSystem<this> | undefined;
    /**
     * Whether or not this room has been destroyed.
     */
    protected _destroyed: boolean;
    protected _started: boolean;
    protected last_fixed_update: number;
    protected _interval?: NodeJS.Timeout;
    spawnPrefabs: NetworkableConstructor<any>[][];
    constructor(options?: HostableOptions);
    destroy(): void;
    getNextNetId(): number;
    /**
     * The current client in the room.
     */
    get me(): PlayerData<this> | undefined;
    /**
     * The host of the room.
     */
    get host(): PlayerData<this> | undefined;
    /**
     * Whether or not a game has started.
     */
    get started(): boolean;
    get destroyed(): boolean;
    /**
     * Whether or not the current client is the host of the room.
     */
    get amhost(): boolean;
    broadcast(messages: BaseGameDataMessage[], reliable?: boolean, recipient?: PlayerData | undefined, payloads?: BaseRootMessage[]): Promise<void>;
    FixedUpdate(): Promise<void>;
    /**
     * Resolve a player by some identifier.
     * @param player The identifier to resolve to a player.
     * @returns The resolved player.
     * @example
     *```typescript
     * // Resolve a player by their clientid.
     * const player = room.resolvePlayer(11013);
     * ```
     */
    resolvePlayer(player: PlayerDataResolvable): PlayerData<this> | undefined;
    /**
     * Resolve a player ID by some identifier.
     * @param player The identifier to resolve to a player ID.
     * @returns The resolved player ID.
     */
    resolvePlayerId(player: PlayerIDResolvable): number | undefined;
    /**
     * Resolve a clientid by some identifier.
     * @param player The identifier to resolve to a client ID.
     * @returns The resolved client ID.
     */
    resolvePlayerClientID(player: PlayerDataResolvable): number | undefined;
    /**
     * Set the code of the room.
     * @example
     *```typescript
     * room.setCode("ABCDEF");
     * ```
     */
    setCode(code: RoomID): void;
    protected _setPrivacy(privacy: PrivacyType): void;
    /**
     * Change the the privacy of the room.
     * @param tag The tag to change.
     * @param value The new value of the tag.
     * @example
     *```typescript
     * room.setAlterGameTag(AlterGameTag.ChangePrivacy, 1); // 0 for private, 1 for public.
     * ```
     */
    setPrivacy(privacy: PrivacyType): Promise<void>;
    /**
     * Change the settings of the room. If the host, it will broadcast these changes.
     * @param settings The settings to set to (Can be partial).
     * @example
     *```typescript
     * room.syncSettings({
     *   crewmateVision: 0.5,
     *   votingTime: 120
     * });
     * ```
     */
    setSettings(settings: Partial<AllGameSettings>): void;
    /**
     * Set the host of the room. If the current client is the host, it will conduct required host changes.
     * e.g. Spawning objects if they are not already spawned.
     * @param host The new host of the room.
     */
    setHost(host: PlayerDataResolvable): Promise<void>;
    /**
     * Handle when a client joins the game.
     * @param clientid The ID of the client that joined the game.
     */
    handleJoin(clientid: number): Promise<PlayerData<this> | null>;
    /**
     * Handle when a client leaves the game.
     * @param resolvable The client that left the game.
     */
    handleLeave(resolvable: PlayerDataResolvable): Promise<PlayerData<this> | null>;
    /**
     * Handle when the game is started.
     */
    handleStart(): Promise<void>;
    /**
     * Start a game.
     */
    requestStartGame(): Promise<void>;
    private _endGame;
    /**
     * Handle when the game is ended.
     * @param reason The reason for why the game ended.
     */
    handleEnd(reason: GameOverReason): Promise<void>;
    /**
     * End the current game.
     */
    endGame(reason: GameOverReason): Promise<void>;
    /**
     * Handle a client readying up.
     * @param player The client that readied.
     */
    handleReady(player: PlayerDataResolvable): Promise<void>;
    /**
     * Spawn a component (Not broadcasted to all clients, see {@link Hostable.spawnPrefab}).
     * @param component The component being spawned.
     * @example
     * ```typescript
     * const meetinghud = new MeetingHud(
     *   this,
     *   this.getNextNetId(),
     *   ownerid,
     *   {
     *     dirtyBit: 0,
     *     states: new Map(),
     *   }
     * );
     *
     * this.spawnComponent(meetinghud);
     * ```
     */
    spawnComponent(component: Networkable<any, NetworkableEvents, this>): void;
    private _despawnComponent;
    /**
     * Despawn a component.
     * @param component The component being despawned.
     * @example
     *```typescript
     * room.despawnComponent(room.meetinghud);
     * ```
     */
    despawnComponent(component: Networkable<any, NetworkableEvents, this>): void;
    /**
     * Get an available player ID.
     * @returns The player ID that was found.
     * @example
     *```typescript
     * // Get an available player ID and add it to the gamedata.
     * const playerId = room.getAvailablePlayerID();
     * room.gamedata.add(playerId);
     * ```
     */
    getAvailablePlayerID(): number;
    /**
     * Spawn a prefab of an object.
     * @param spawnType The type of object to spawn.
     * @param owner The owner or ID of the owner of the object to spawn.
     * @returns The object that was spawned.
     * @example
     *```typescript
     * room.spawnPrefab(SpawnType.Player, client.me);
     * ```
     */
    spawnPrefab(spawnType: number, ownerid: number | {
        id: number;
    }, flags?: number, componentData?: ComponentSpawnData[], doBroadcast?: boolean): Networkable<any, NetworkableEvents<Hostable<any>>, Hostable<any>>;
    registerPrefab(spawnType: number, components: NetworkableConstructor<Networkable>[]): void;
    /**
     * Get a player by their player ID.
     * @param playerId The player ID of the player.
     * @returns The player that was found, or null if they do not exist.
     * @example
     * ```typescript
     * const player = room.getPlayerByPlayerId(1);
     * ```
     */
    getPlayerByPlayerId(playerId: number): PlayerData<this> | undefined;
    /**
     * Get a player by one of their components' netids.
     * @param netid The network ID of the component of the player to search.
     * @returns The player that was found, or null if they do not exist.
     * @example
     * ```typescript
     * const player = room.getPlayerByNetId(34);
     * ```
     */
    getPlayerByNetId(netid: number): PlayerData<this> | undefined;
    private _getExistingObjectSpawn;
    /**
     * How often a FixedUpdate should be called.
     */
    static FixedUpdateInterval: number;
}
